/* Parsing FTP `ls' output.
   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.

This file is part of Wget.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_STRING_H
# include <string.h>
#else
# include <strings.h>
#endif
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#include <sys/types.h>
#include <ctype.h>
#include <errno.h>

#include "wget.h"
#include "utils.h"
#include "ftp.h"
#include "ftpparse.h"

/* Convert the Un*x-ish style directory listing stored in FILE to a
   linked list of fileinfo (system-independent) entries.  The contents
   of FILE are considered to be produced by the standard Unix `ls -la'
   output (whatever that might be).  BSD (no group) and SYSV (with
   group) listings are handled.

   The time stamps are stored in a separate variable, time_t
   compatible (I hope).  The timezones are ignored.  */
static struct fileinfo *
ftp_parse_unix_ls (const char *file)
{
  FILE *fp;
  int len;

  char *line;          /* tokenizer */
  struct fileinfo *dir, *l, cur; /* list creation */

  fp = fopen (file, "rb");
  if (!fp)
    {
      logprintf (LOG_NOTQUIET, "%s: %s\n", file, strerror (errno));
      return NULL;
    }
  dir = l = NULL;

  /* Line loop to end of file: */
  while ((line = read_whole_line (fp)))
    {
           struct ftpparse fp;

      DEBUGP (("%s\n", line));
      len = strlen (line);
      /* Destroy <CR> if there is one.  */
      if (len && line[len - 1] == '\r')
       line[--len] = '\0';

      if (ftpparse(&fp, line, len)) {
             cur.size = fp.size;
             cur.name = (char *)xmalloc (fp.namelen + 1);
             memcpy (cur.name, fp.name, fp.namelen);
             cur.name[fp.namelen] = '\0';
             DEBUGP (("%s\n", cur.name));
             cur.linkto = NULL;
             if (fp.flagtrycwd) {
                     cur.type = FT_DIRECTORY;
                     cur.perms = 0644;
             } else {
                     cur.type = FT_PLAINFILE;
                     cur.perms = 0755;
             }
             if (!dir) {
                     l = dir = (struct fileinfo *)xmalloc (sizeof (struct fileinfo));
                     memcpy (l, &cur, sizeof (cur));
                     l->prev = l->next = NULL;
             } else {
                     cur.prev = l;
                     l->next = (struct fileinfo *)xmalloc (sizeof (struct fileinfo));
                     l = l->next;
                     memcpy (l, &cur, sizeof (cur));
                     l->next = NULL;
             }
             l->tstamp = fp.mtime;
      }

      free (line);
    }

  fclose (fp);
  return dir;
}

/* This function is just a stub.  It should actually accept some kind
   of information what system it is running on -- e.g. FPL_UNIX,
   FPL_DOS, FPL_NT, FPL_VMS, etc. and a "guess-me" value, like
   FPL_GUESS.  Then it would call the appropriate parsers to fill up
   fileinfos.

   Since we currently support only the Unix FTP servers, this function
   simply returns the result of ftp_parse_unix_ls().  */
struct fileinfo *
ftp_parse_ls (const char *file)
{
  return ftp_parse_unix_ls (file);
}
